﻿#nullable disable

using static Paysagiste.PartieProfs;

namespace Paysagiste
{
   static class PartieProfs
   {
      public static void AfficherSymbole(char c)
      {
         if(Couleurs.ContainsKey(c))
         {
            ConsoleColor pre = Console.ForegroundColor;
            Console.ForegroundColor = Couleurs[c];
            Console.Write(c);
            Console.ForegroundColor = pre;
         }
         else
         {
            Console.Write(c);
         }
      }
      static Dictionary<char, ConsoleColor> Couleurs { get; } = new();
      public static void AfficherOptionMenu(OptionMenu opt)
      {
         Console.Write($"{opt} : {(int)opt}");
      }
      public static void AfficherMenu()
      {
         OptionMenu[] opts = new OptionMenu[]
         {
            OptionMenu.Décrire, OptionMenu.Avancer, OptionMenu.Reculer,
            OptionMenu.AjouterForme, OptionMenu.SupprimerForme,
            OptionMenu.Sauvegarder, OptionMenu.Récupérer,
            OptionMenu.Déplacer,
            OptionMenu.Vider, OptionMenu.Quitter
         };
         Console.Write("| ");
         for (int i = 0; i != opts.Length; ++i)
         {
            AfficherOptionMenu(opts[i]);
            Console.Write(" | ");
         }
         Console.WriteLine();
      }
      public static OptionMenu LireOptionMenu()
      {
         return (OptionMenu)PartieÉtudiante.LireEntier("Votre choix? ", (int)OptionMenu.Décrire, (int)OptionMenu.Quitter);
      }

      public static Point LirePoint(Paysage paysage)
      {
         return PartieÉtudiante.LirePoint(paysage.ObtenirCanevas());
      }

      public static Carré LireCarré(Paysage paysage)
      {
         return PartieÉtudiante.LireCarré(paysage.ObtenirCanevas());
      }

      public static Rectangle LireRectangle(Paysage paysage)
      {
         return PartieÉtudiante.LireRectangle(paysage.ObtenirCanevas());
      }

      public static Triangle LireTriangle(Paysage paysage)
      {
         return PartieÉtudiante.LireTriangle(paysage.ObtenirCanevas());
      }

      public static void AjouterForme(Paysage paysage)
      {
         PartieÉtudiante.AfficherSortesFormes();
         PartieÉtudiante.SorteForme sorte = PartieÉtudiante.LireSorteForme();
         Forme forme = null;
         switch (sorte)
         {
            case PartieÉtudiante.SorteForme.Carré:
               {
                  Carré carré = LireCarré(paysage);
                  while (!PartieÉtudiante.EstDansCanevas(paysage.ObtenirCanevas(), carré))
                  {
                     Console.Write("Cette forme déborde du canevas, recommencez : ");
                     carré = LireCarré(paysage);
                  }
                  forme = carré;
               }
               break;
            case PartieÉtudiante.SorteForme.Rectangle:
               {
                  Rectangle rect = LireRectangle(paysage);
                  while (!PartieÉtudiante.EstDansCanevas(paysage.ObtenirCanevas(), rect))
                  {
                     Console.Write("Cette forme déborde du canevas, recommencez : ");
                     rect = LireRectangle(paysage);
                  }
                  forme = rect;
               }
               break;
            case PartieÉtudiante.SorteForme.Triangle:
               {
                  Triangle triangle = LireTriangle(paysage);
                  while (!PartieÉtudiante.EstDansCanevas(paysage.ObtenirCanevas(), triangle))
                  {
                     Console.Write("Cette forme déborde du canevas, recommencez : ");
                     triangle = LireTriangle(paysage);
                  }
                  forme = triangle;
               }
               break;
         }
         paysage.Ajouter(forme);
         Colorier(forme);
      }

      static bool EstColoriée(Forme forme) =>
         Couleurs.ContainsKey(forme.Symbole);
      static void Colorier(Forme forme)
      {
         if(EstColoriée(forme))
         {
            Console.WriteLine("Il y a déjà une couleur associée à ce symbole");
         }
         else
         {
            Couleurs.Add(forme.Symbole, LireCouleur());
         }
      }
      static ConsoleColor LireCouleur()
      {
         ConsoleColor[] couleurs =
         {
            ConsoleColor.Red, ConsoleColor.Green, ConsoleColor.Blue,
            ConsoleColor.Magenta, ConsoleColor.Yellow, ConsoleColor.Cyan,
            ConsoleColor.Gray, ConsoleColor.White
         };
         Console.WriteLine("Quelle couleur?");
         Console.Write("|");
         for(int i = 0; i != couleurs.Length; ++i)
         {
            ConsoleColor pre = Console.ForegroundColor;
            Console.ForegroundColor = couleurs[i];
            Console.Write($" {i} : {couleurs[i]} |");
            Console.ForegroundColor = pre;
         }
         Console.Write("Votre choix? ");
         int choix = int.Parse(Console.ReadLine());
         while(choix < 0 || couleurs.Length <= choix)
         {
            Console.Write("Votre choix? ");
            choix = int.Parse(Console.ReadLine());
         }
         return couleurs[choix];
      }

      public static void Décrire(Paysage paysage)
      {
         Console.WriteLine($"Canevas de hauteur {paysage.Hauteur} et de largeur {paysage.Largeur}");
         PartieÉtudiante.DécrirePaysage(paysage.Décrire());
      }

      public static int ChoisirForme(Paysage paysage)
      {
         return PartieÉtudiante.ChoisirForme(paysage.Éléments.ToArray());
      }

      public static void ChangerProfondeurForme(Paysage paysage, int laquelle, int dp)
      {
         paysage.MettreÀJourÉléments
         (
            PartieÉtudiante.ChangerProfondeur
            (
               paysage.Éléments.ToArray(), laquelle, dp
            )
         );
      }

      public static void SupprimerForme(Paysage paysage, int laquelle)
      {
         if (PartieÉtudiante.EstProfondeurValide(paysage.Éléments.ToArray(), laquelle))
         {
            paysage.Éléments.RemoveAt(laquelle);
         }
      }

      public static void AvancerForme(Paysage paysage)
      {
         Décrire(paysage);
         int laquelle = ChoisirForme(paysage);
         if (laquelle != -1)
         {
            ChangerProfondeurForme(paysage, laquelle, 1);
         }
      }

      public static void ReculerForme(Paysage paysage)
      {
         Décrire(paysage);
         int laquelle = ChoisirForme(paysage);
         if (laquelle != -1)
         {
            ChangerProfondeurForme(paysage, laquelle, -1);
         }
      }

      public static void SupprimerForme(Paysage paysage)
      {
         Décrire(paysage);
         int laquelle = ChoisirForme(paysage);
         if (laquelle != -1)
         {
            SupprimerForme(paysage, laquelle);
         }
      }

      public static void Sauvegarder(Paysage paysage)
      {
         Console.Write("Nom du fichier pour sauvegarde : ");
         string nom = Console.ReadLine();
         using (var sw = File.CreateText(nom))
         {
            sw.WriteLine($"%% créé le {System.DateTime.Now}");
            sw.WriteLine($"Hauteur : {paysage.Hauteur} Largeur : {paysage.Largeur}");
            foreach (var (c, coul) in Couleurs)
               sw.WriteLine($"{c} {coul}");
            sw.WriteLine(); // fin des métadonnées
            foreach (string s in paysage.Décrire())
               sw.WriteLine(s);
         }
      }

      public static void Récupérer(ref Paysage paysage)
      {
         Console.Write("Nom du fichier à récupérer : ");
         string nom = Console.ReadLine();
         if (!System.IO.File.Exists(nom))
         {
            Console.WriteLine($"Fichier {nom} inexistant");
         }
         else
         {
            using (var sr = File.OpenText(nom))
            {
               Console.WriteLine(sr.ReadLine()); // commentaire sur la date
               string[] strs = sr.ReadLine().Split(new char[] { ' ' }).Where(s => s.Length != 0).ToArray();
               // Hauteur : hau Largeur : lar
               int hauteur = int.Parse(strs[2]),
                   largeur = int.Parse(strs[5]);
               Console.WriteLine($"Trouvé un canevas de hauteur {hauteur} et de largeur {largeur}");
               paysage = new Paysage(hauteur, largeur);
               Couleurs.Clear();
               for(string s = sr.ReadLine(); s.Length != 0; s = sr.ReadLine())
               {
                  char c = s[0];
                  ConsoleColor coul = (ConsoleColor) Enum.Parse(typeof(ConsoleColor), s.Substring(2));
                  Couleurs.Add(c, coul);
               }
               for (string s = sr.ReadLine(); s != null; s = sr.ReadLine())
               {
                  strs = s.Split(new char[] { ' ', ',', '(', ')', ';' }).Where(str => str.Length != 0).ToArray();
                  if (strs[0].Equals("Carré"))
                  {
                     paysage.Ajouter(new Carré(
                        char.Parse(strs[2]), // symbole
                        new Point(int.Parse(strs[4]), int.Parse(strs[5])), // x,y
                        int.Parse(strs[7]) // hauteur
                     ));
                  }
                  else if (strs[0].Equals("Rectangle"))
                  {
                     paysage.Ajouter(new Rectangle(
                        char.Parse(strs[2]), // symbole
                        new Point(int.Parse(strs[4]), int.Parse(strs[5])), // x,y
                        int.Parse(strs[7]), // hauteur
                        int.Parse(strs[9]) // largeur
                     ));
                  }
                  else if (strs[0].Equals("Triangle"))
                  {
                     paysage.Ajouter(new Triangle(
                        char.Parse(strs[2]), // symbole
                        new Point(int.Parse(strs[4]), int.Parse(strs[5])), // x,y
                        int.Parse(strs[7]) // hauteur
                     ));
                  }
                  else
                  {
                     Console.WriteLine($"Fichier apparemment corrompu (cherche une forme, trouve {strs[0]}");
                  }
               }
            }
         }
      }

      public static bool EstDansCanevas(Paysage paysage, Forme forme)
      {
         return forme.EstDansCanevas(paysage);
      }

      public static void Déplacer(ref Paysage paysage)
      {
         Décrire(paysage);
         int laquelle = ChoisirForme(paysage);
         if (laquelle != -1)
         {
            Console.Clear();
            PartieÉtudiante.Afficher(paysage.ObtenirCanevas());
            Console.WriteLine("Utilisez les flèches pour déplacer, et 'Échap' pour arrêter");
            ConsoleKeyInfo touche = Console.ReadKey(true);
            while (touche.Key != ConsoleKey.Escape)
            {
               bool doitBiper = false;
               int dx = 0,
                   dy = 0;
               switch (touche.Key)
               {
                  case ConsoleKey.RightArrow:
                     dx = 1;
                     break;
                  case ConsoleKey.UpArrow:
                     dy = -1;
                     break;
                  case ConsoleKey.LeftArrow:
                     dx = -1;
                     break;
                  case ConsoleKey.DownArrow:
                     dy = 1;
                     break;
                  default:
                     doitBiper = true;
                     break;
               }
               Forme candidate = paysage.Éléments[laquelle].ClonerTranslaté(dx, dy);
               if (candidate.EstDansCanevas(paysage))
               {
                  paysage.Éléments[laquelle] = candidate;
               }
               else
               {
                  doitBiper = true;
               }
               if (doitBiper)
               {
                  Console.Beep();
               }
               Console.Clear();
               PartieÉtudiante.Afficher(paysage.ObtenirCanevas());
               Console.WriteLine("Utilisez les flèches pour déplacer, et 'Échap' pour arrêter");
               touche = Console.ReadKey(true);
            }
            Console.Clear();
         }
      }

      public static Paysage LirePaysage()
      {
         int hauteur = PartieÉtudiante.LireEntier("Hauteur du canevas? ", 3, Paysage.HAUTEUR_MAX);
         int largeur = PartieÉtudiante.LireEntier("Largeur du canevas? ", 3, Paysage.LARGEUR_MAX);
         return new Paysage(hauteur, largeur);
      }

      public static void Traiter(OptionMenu choix, ref Paysage paysage)
      {
         switch (choix)
         {
            case OptionMenu.AjouterForme:
               AjouterForme(paysage);
               break;
            case OptionMenu.Décrire:
               Décrire(paysage);
               break;
            case OptionMenu.Avancer:
               AvancerForme(paysage);
               break;
            case OptionMenu.Reculer:
               ReculerForme(paysage);
               break;
            case OptionMenu.SupprimerForme:
               SupprimerForme(paysage);
               break;
            case OptionMenu.Sauvegarder:
               Sauvegarder(paysage);
               break;
            case OptionMenu.Récupérer:
               Récupérer(ref paysage);
               break;
            case OptionMenu.Déplacer:
               Déplacer(ref paysage);
               break;
            case OptionMenu.Vider:
               paysage = LirePaysage();
               break;
            default:
               Console.WriteLine($"Vous avez choisi {choix}");
               break;
         }
      }

      public enum OptionMenu
      {
         Décrire,
         Avancer,
         Reculer,
         AjouterForme,
         SupprimerForme,
         Sauvegarder,
         Récupérer,
         Déplacer,
         Vider,
         Quitter
      }




      public class Point : IEquatable<Point>
      {
         public int X { get; init; }
         public int Y { get; init; }
         public Point(int x, int y)
         {
            X = x;
            Y = y;
         }
         public bool Equals(Point autre) => X == autre.X && Y == autre.Y;
         public static bool operator ==(Point p0, Point p1) =>
            p0.Equals(p1);
         public static bool operator !=(Point p0, Point p1) => !(p0 == p1);
         public override bool Equals(object obj) =>
            obj is Point pt && Equals(pt);
         public override int GetHashCode() => base.GetHashCode(); // bof
      }
      public abstract class Forme
      {
         public Point Position { get; init; }
         public char Symbole { get; init; }
         public abstract string Nom { get; init; }
         public Forme(char symbole, Point position)
         {
            Symbole = symbole;
            Position = position;
         }
         public abstract void Dessiner(char[,] canevas);
         public abstract string Décrire();
         public abstract bool EstDansCanevas(Paysage paysage);
         public abstract Forme ClonerTranslaté(int dx, int dy);
      }

      //
      //
      //
      public class Triangle : Forme, IEquatable<Triangle>
      {
         public int Hauteur { get; private set; }
         public override string Nom { get; init; } = "Triangle";
         public Triangle(char symbole, Point position, int hauteur)
            : base(symbole, position)
         {
            Hauteur = hauteur;
         }
         public override void Dessiner(char[,] canevas)
         {
            PartieÉtudiante.DessinerTriangle(canevas, Symbole, Position, Hauteur);
         }
         public override string Décrire() =>
            $"{Nom}, symbole {Symbole}, position ({Position.X};{Position.Y}), hauteur {Hauteur}";
         public override bool EstDansCanevas(Paysage paysage)
         {
            return PartieÉtudiante.EstDansCanevas(paysage.ObtenirCanevas(), this);
         }
         public override Forme ClonerTranslaté(int dx, int dy) =>
            new Triangle(Symbole, new Point(Position.X + dx, Position.Y + dy), Hauteur);
         public bool Equals(Triangle autre) =>
            Hauteur == autre.Hauteur &&
            Position == autre.Position &&
            Symbole == autre.Symbole;
         public static bool operator ==(Triangle t0, Triangle t1) =>
            t0.Equals(t1);
         public static bool operator !=(Triangle t0, Triangle t1) =>
            !(t0 == t1);
         public override bool Equals(object autre) =>
            autre is Triangle t && Equals(t);
         public override int GetHashCode() => base.GetHashCode(); // bof
      }
      public class Carré : Forme, IEquatable<Carré>
      {
         public int Hauteur { get; init; }
         public override string Nom { get; init; } = "Carré";
         public Carré(char symbole, Point position, int hauteur)
            : base(symbole, position)
         {
            Hauteur = hauteur;
         }
         public override void Dessiner(char[,] canevas)
         {
            PartieÉtudiante.DessinerCarré(canevas, Symbole, Position, Hauteur);
         }
         public override string Décrire() =>
            $"{Nom}, symbole {Symbole}, position ({Position.X};{Position.Y}), hauteur {Hauteur}";
         public override bool EstDansCanevas(Paysage paysage)
         {
            return PartieÉtudiante.EstDansCanevas(paysage.ObtenirCanevas(), this);
         }
         public override Forme ClonerTranslaté(int dx, int dy) =>
            new Carré(Symbole, new Point(Position.X + dx, Position.Y + dy), Hauteur);
         public bool Equals(Carré autre) =>
            Hauteur == autre.Hauteur &&
            Position == autre.Position &&
            Symbole == autre.Symbole;
         public static bool operator ==(Carré c0, Carré c1) => c0.Equals(c1);
         public static bool operator !=(Carré c0, Carré c1) => !(c0 == c1);
         public override bool Equals(object autre) =>
            autre is Carré c && Equals(c);
         public override int GetHashCode() => base.GetHashCode(); // bof
      }
      public class Rectangle : Forme, IEquatable<Rectangle>
      {
         public int Hauteur { get; init; }
         public int Largeur { get; init; }
         public override string Nom { get; init; } = "Rectangle";
         public Rectangle(char symbole, Point position, int hauteur, int largeur)
            : base(symbole, position)
         {
            Hauteur = hauteur;
            Largeur = largeur;
         }
         public override void Dessiner(char[,] canevas)
         {
            PartieÉtudiante.DessinerRectangle(canevas, Symbole, Position, Hauteur, Largeur);
         }
         public override string Décrire() =>
            $"{Nom}, symbole {Symbole}, position ({Position.X};{Position.Y}), hauteur {Hauteur}, largeur {Largeur}";
         public override bool EstDansCanevas(Paysage paysage)
         {
            return PartieÉtudiante.EstDansCanevas(paysage.ObtenirCanevas(), this);
         }
         public override Forme ClonerTranslaté(int dx, int dy) =>
            new Rectangle(Symbole, new Point(Position.X + dx, Position.Y + dy), Hauteur, Largeur);
         public bool Equals(Rectangle autre) =>
            Hauteur == autre.Hauteur &&
            Largeur == autre.Largeur &&
            Position == autre.Position &&
            Symbole == autre.Symbole;

         public static bool operator ==(Rectangle r0, Rectangle r1) =>
            r0.Equals(r1);
         public static bool operator !=(Rectangle r0, Rectangle r1) =>
            !(r0 == r1);
         public override bool Equals(object autre) =>
            autre is Rectangle r && Equals(r);
         public override int GetHashCode() => base.GetHashCode(); // bof
      }
      public class Paysage
      {
         public int Hauteur { get; init; } = HAUTEUR_MAX;
         public int Largeur { get; init; } = LARGEUR_MAX;

         public Paysage(int hauteur, int largeur)
         {
            Hauteur = hauteur;
            Largeur = largeur;
         }

         public List<Forme> Éléments { get; private set; } = new();
         public void MettreÀJourÉléments(Forme[] formes)
         {
            Éléments = formes.ToList();
         }
         public const int LARGEUR_MAX = 100,
                          HAUTEUR_MAX = 25;
         void Dessiner(char[,] canevas)
         {
            foreach (Forme f in Éléments)
            {
               f.Dessiner(canevas);
            }
         }
         private bool EstSurCadre(int x, int y) =>
            x == 0 || x == Largeur - 1 || y == 0 || y == Hauteur - 1;
         public char[,] ObtenirCanevas()
         {
            char[,] canevas = new char[Hauteur, Largeur];
            for (int ligne = 0; ligne != canevas.GetLength(0); ++ligne)
            {
               for (int colonne = 0; colonne != canevas.GetLength(1); ++colonne)
               {
                  if (EstSurCadre(colonne, ligne))
                  {
                     canevas[ligne, colonne] = '#';
                  }
                  else
                  {
                     canevas[ligne, colonne] = ' ';
                  }
               }
            }
            Dessiner(canevas);
            return canevas;
         }
         public void Ajouter(Forme forme)
         {
            Éléments.Add(forme);
         }
         public string[] Décrire()
         {
            string[] descriptions = new string[Éléments.Count];
            for (int i = 0; i != Éléments.Count; ++i)
            {
               descriptions[i] = Éléments[i].Décrire();
            }
            return descriptions;
         }
      }
   }
}
