﻿#nullable disable
using static Paysagiste.PartieProfs;

namespace Paysagiste
{
   class PartieÉtudiante
   {
      // -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
      // -- Concepts généraux
      // -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-

      //
      // Canevas :
      // - Certaines fonctions utilisent un canevas, qui est un tableau 2D rectangulaire
      //   de char. Les Point et les diverses formes doivent se situer en entier à l'intérieur
      //   du canevas
      // - Par exemple, pour un canevas d'une hauteur H et d'une largeur L, les positions
      //   0 et H-1 sur l'axe des y sont réservées pour le cadre du canevas, et il en va de
      //   même pour les positions 0 et L-1 en x
      // - Le système d'axes d'un canevas n'est pas un système cartésien classique, en ce
      //   sens que le point à la position 0,0 est en haut et à gauche. Les coordonnées
      //   vont en croissant vers le bas (hauteur) et vers la droite (largeur)
      // - Par exemple, le canevas suivant contient deux formes, soit un Rectangle de
      //   hauteur 2 et de largeur 5 dessiné à l'aide du symbole % et situé à la position
      //   x:3,y:3 et un Triangle de hauteur 3 dessiné à l'aide du symbole $ et situé à la
      //   position x:5,y:6 le tout sur un canevas de hauteur 10 et de largeur 20 :
      //
      // ####################
      // #                  #
      // #                  #
      // #  %%%%%           #
      // #  %%%%%           #
      // #                  #
      // #      $           #
      // #     $$$          #
      // #    $$$$$         #
      // ####################
      // 
      // Profondeur :
      // - Dans un canevas, des formes sont dessinées. Celles-ci sont dans un tableau,
      //   ordonnancées de la plus profonde à la moins profonde
      // - Ainsi, pour dessiner sur le canevas, la forme à la profondeur 0 sera dessinée
      //   d'abord, puis celle à la profondeur 1, puis celle à la profondeur 2, etc.
      //
      // Point :
      // - Un Point modélise une position sur le canevas
      //

      // -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
      // - Sortes de formes
      // -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-

      //
      // Énumération SorteForme
      // - liste symboliquement les sortes de formes supportées par le logiciel
      // - doit contenir Carré, Rectangle et Triangle
      //


      //
      // Fonction AfficherSortesFormes()
      // Intrants :
      // - aucun
      // Extrant :
      // - aucun
      // Rôle : afficher chaque sorte de forme accompagnée de sa valeur entière
      //        (p. ex. : "Carré : 0")
      // Note :
      // - les sortes de formes doivent être délimitées à l'affichage par le symbole
      //   '|', comme par exemple "| Carré : 0 | Rectangle : 1 | ... etc."
      //


      //
      // Fonction LireSorteForme()
      // Intrants :
      // - aucun
      // Extrant :
      // - la sorte de forme lue et validée
      // Rôle : lire et valider une sorte de forme
      //
      public static SorteForme LireSorteForme()
      {
         // Oups!
      }


      // -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
      // -- Outils d'entrée / sortie
      // -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-

      //
      // Fonction LireEntier(invite,min,max)
      // Intrants :
      // - invite : message expliquant la nature de l'entrée à réaliser. Sera affiché
      // - min : borne minimale des valeurs acceptables en entrée
      // - max : borne maximale des valeurs acceptables en entrée
      // Note : les bornes sont inclusives; l'intervalle acceptable est [min,max]
      // Extrant :
      // - la valeur lue et validée
      // Rôle : lire et valider un entier devant se situer dans [min,max]
      //

      //
      // Fonction LirePoint(canevas)
      // Intrant :
      // - canevas : l'espace dans lequel le Point lu devra se situer
      // Note : les contours du canevas ne sont pas des endroits acceptables pour
      //        un Point tel que lu par cette fonction. Voir Concepts généraux,
      //        plus haut, pour des détails
      // Extrant :
      // - le Point lu et validé
      // Rôle : lire et valider un Point devant se situer dans le canevas
      //
      public static Point LirePoint(char[,] canevas)
      {
         // Oups!
      }


      //
      // Fonction LireSymbole()
      // Intrants :
      // - aucun
      // Extrant :
      // - le symbole lu et validé
      // Rôle : lire un symbole qui sera utilisé pour fins d'affichage à l'écran
      // Note : tout symbole (tout char) sera acceptable. L'usager est responsable
      //        de ne pas entrer un symbole non-affichable ou un blanc
      //


      //
      // Fonction LireCarré(canevas)
      // Intrant :
      // - canevas : l'espace dans lequel le Carré lu devra se situer
      // Note : le Carré lu doit se situer en entier à l'intérieur du canevas (voir
      //        Concepts généraux, plus haut), mais cette validation ne sera faite
      //        que partiellement par LireCarré(), et sera complétée ultérieurement
      //        par EstDansCanevas(canevas,Carré)
      // Extrant :
      // - le Carré lu
      // Rôle : lire et valider un Carré dont la position se situe dans le canevas et
      //        dont la hauteur ne dépasse pas celle du canevas
      //


      //
      // Fonction LireRectangle(canevas)
      // Intrant :
      // - canevas : l'espace dans lequel le Rectangle lu devra se situer
      // Note : le Rectangle lu doit se situer en entier à l'intérieur du canevas (voir
      //        Concepts généraux, plus haut), mais cette validation ne sera faite
      //        que partiellement par LireRectangle(), et sera complétée ultérieurement
      //        par EstDansCanevas(canevas,Rectangle)
      // Extrant :
      // - le Rectangle lu
      // Rôle : lire et valider un Rectangle dont la position se situe dans le canevas et
      //        dont la hauteur et la largeur ne dépassent pas celles du canevas
      //


      //
      // Fonction LireTriangle(canevas)
      // Intrant :
      // - canevas : l'espace dans lequel le Triangle lu devra se situer
      // Note : le Triangle lu doit se situer en entier à l'intérieur du canevas (voir
      //        Concepts généraux, plus haut), mais cette validation ne sera faite
      //        que partiellement par LireTriangle(), et sera complétée ultérieurement
      //        par EstDansCanevas(canevas,Triangle)
      // Extrant :
      // - le Triangle lu
      // Rôle : lire et valider un Triangle dont la position se situe dans le canevas et
      //        dont la hauteur ne dépasse pas celle du canevas
      //
      public static Triangle LireTriangle(char[,] canevas)
      {
         // Oups!
      }


      //
      // Fonction ChoisirForme(formes)
      // Intrants :
      // - formes : un tableau de Forme contenant les formes destinées au canevas
      // Extrant :
      // - l'indice de la forme choisie
      // Rôle :
      // - lire et valider un indice de forme valide pour le tableau formes, ou -1 si
      //   le tableau est de taille nulle (zéro)
      // Note : le type Forme représente une Forme au sens large, mais vous n'avez pas
      //        besoin d'en connaître le détail
      //
      public static int ChoisirForme(Forme[] formes)
      {
         // Oups!
      }

      // -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
      // -- Prédicats utilitaires
      // -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-

      //
      // Fonction EstEntreInclusif(valeur,min,max)
      // Intrants :
      // - valeur : valeur dont on souhaite valider l'appartenance à un intervalle
      // - min : borne minimale de l'intervalle
      // - max : borne maximale de l'intervalle
      // Note : les bornes sont inclusives; l'intervalle acceptable est [min,max]
      // Extrant :
      // - vrai seulement si la valeur est dans l'intervalle [min,max] (donc faux sinon)
      //


      //
      // Fonction EstProfondeurValide(formes,profondeur)
      // Intrants :
      // - formes : un tableau des formes destinées au canevas
      // - profondeur : la profondeur candidate
      // Extrant :
      // - vrai seulement si la profondeur candidate est une position valide dans le
      //   tableau formes (donc faux sinon)
      // Note : le type Forme représente une Forme au sens large, mais vous n'avez pas
      //        besoin d'en connaître le détail
      //
      public static bool EstProfondeurValide(Forme[] formes, int profondeur)
      {
         // Oups!
      }

      //
      // Fonction EstDansCanevas(canevas,point)
      // Intrants :
      // - canevas : l'espace dans lequel le Point reçu en paramètre devra se situer
      // - point : le Point candidat
      // Note : les contours du canevas ne sont pas des endroits acceptables pour
      //        un Point tel que celui reçu en paramètre par cette fonction. Voir
      //        Concepts généraux, plus haut, pour des détails
      // Extrant :
      // - vrai seulement si le point est dans le canevas (donc faux sinon)
      //


      //
      // Fonction EstDansCanevas(canevas,carré)
      // Intrants :
      // - canevas : l'espace dans lequel le Carré reçu en paramètre devra se situer
      // - carré : le Carré candidat
      // Note : les contours du canevas ne sont pas des endroits acceptables pour
      //        un Carré tel que celui reçu en paramètre par cette fonction. Voir
      //        Concepts généraux, plus haut, pour des détails
      // Extrant :
      // - vrai seulement si le carré est entièrement dans le canevas (donc faux sinon)
      //


      //
      // Fonction EstDansCanevas(canevas,rectangle)
      // Intrants :
      // - canevas : l'espace dans lequel le Rectangle reçu en paramètre devra se situer
      // - rectangle : le Rectangle candidat
      // Note : les contours du canevas ne sont pas des endroits acceptables pour
      //        un Rectangle tel que celui reçu en paramètre par cette fonction. Voir
      //        Concepts généraux, plus haut, pour des détails
      // Extrant :
      // - vrai seulement si le rectangle est entièrement dans le canevas (donc faux sinon)
      //
      public static bool EstDansCanevas(char[,] canevas, Rectangle rectangle)
      {
         // Oups!
      }


      //
      // Fonction EstDansCanevas(canevas,triangle)
      // Intrants :
      // - canevas : l'espace dans lequel le Triangle reçu en paramètre devra se situer
      // - triangle : le Triangle candidat
      // Note : les contours du canevas ne sont pas des endroits acceptables pour
      //        un Triangle tel que celui reçu en paramètre par cette fonction. Voir
      //        Concepts généraux, plus haut, pour des détails
      // Extrant :
      // - vrai seulement si le triangle est entièrement dans le canevas (donc faux sinon)
      //


      // -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
      // -- Fonctions de dessin
      // -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-

      //
      // Fonction DessinerCarré(canevas,symbole,position,hauteur)
      // Intrants :
      // - canevas : l'espace dans lequel le carré devra se situer
      // - symbole : le symbole à utiliser pour dessiner le carré
      // - position : la position du coin supérieur gauche du carré sur le canevas
      // - hauteur : la hauteur du carré
      // Extrant :
      // - aucun
      // Rôle :
      // - dessiner sur le canevas le carré décrit par les paramètres
      //
      public static void DessinerCarré(char[,] canevas, char symbole, Point position, int hauteur)
      {
         // Oups!
      }

      //
      // Fonction DessinerRectangle(canevas,symbole,position,hauteur,largeur)
      // Intrants :
      // - canevas : l'espace dans lequel le rectangle devra se situer
      // - symbole : le symbole à utiliser pour dessiner le rectangle
      // - position : la position du coin supérieur gauche du rectangle sur le canevas
      // - hauteur : la hauteur du rectangle
      // - largeur : la largeur du rectangle
      // Extrant :
      // - aucun
      // Rôle :
      // - dessiner sur le canevas le rectangle décrit par les paramètres
      //


      //
      // Fonction DessinerTriangle(canevas,symbole,position,hauteur)
      // Intrants :
      // - canevas : l'espace dans lequel le triangle devra se situer
      // - symbole : le symbole à utiliser pour dessiner le triangle
      // - position : la position du coin supérieur gauche du triangle sur le canevas
      // - hauteur : la hauteur du triangle
      // Extrant :
      // - aucun
      // Rôle :
      // - dessiner sur le canevas le triangle décrit par les paramètres
      // Note :
      // - le triangle doit être centré. Ainsi, ce qui suit est un triangle de hauteur
      //   4 dessiné à l'aide du symbole * :
      //        *
      //       ***
      //      *****
      //     *******
      // - l'espace occupé par un triangle est défini par son rectangle englobant le plus
      //   petit. À titre d'illustration, avec le triangle ci-dessus, l'espace occupé par
      //   le triangle est décrit par le rectangle suivant :
      //     ...*...
      //     ..***..
      //     .*****.
      //     *******
      // - la position du triangle sur le canevas est le coin en haut et à gauche de son
      //   rectangle englobant
      //


      //
      // Fonction DécrirePaysage(descriptionFormes)
      // Intrants :
      // - descriptionFormes : tableau de chaînes de caractères décrivant les formes
      //                       destinées à être affichées sur le canevas
      // Note :
      // - les descriptions sont placées en ordre croissant de profondeur, donc de la
      //   plus profonde à la moins profonde
      // Extrant :
      // - aucun
      // Rôle :
      // - afficher les descriptions des formes, en ordre croissant de profondeur, en
      //   prenant soin d'indiquer l'indice de chacune
      // Note :
      // - si descriptionFormes est de taille nulle (zéro), alors cette fonction doit
      //   afficher le texte "Le paysage est désert"
      //


      //
      // Fonction Afficher(canevas)
      // Intrants :
      // - canevas : le canevas à dessiner (un tableau 2D rectangulaire de char)
      // Extrant :
      // - aucun
      // Rôle :
      // - afficher le canevas à l'écran
      // Note :
      // - Pour afficher un char à l'écran, appelez la fonction AfficherSymbole
      //   (elle se trouve dans PartieProfs.cs) plutôt que Console.Write
      //   car cette fonction affichera le caractère en utilisant la
      //   couleur choisie en créant la forme associée au symbole (s'il
      //   y a lieu)
      //


      // -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
      // -- Autres outils
      // -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-

      //
      // Fonction Permuter(f0,f1)
      // Intrants :
      // - f0 : référence sur une Forme
      // - f1 : référence sur une Forme
      // Extrants :
      // - f0 : référence sur une Forme
      // - f1 : référence sur une Forme
      // Rôle :
      // - permuter f0 et f1
      // Note : le type Forme représente une Forme au sens large, mais vous n'avez pas
      //        besoin d'en connaître le détail
      //
      static void Permuter(ref Forme f0, ref Forme f1)
      {
         // Oups!
      }

      //
      // Fonction ChangerProfondeur(formes,laquelle,déplacement)
      // Intrants :
      // - formes : un tableau de Forme
      // - laquelle : indice de la Forme à déplacer
      // - déplacement : déplacement à réaliser
      // Notes :
      // - le déplacement doit être 1 ou -1 inclusivement. Un déplacement d'une autre
      //   valeur ne doit pas avoir d'effet
      // - un déplacement qui amènerait à déborder du tableau formes ne doit pas avoir
      //   d'effet
      // Rôle :
      // - permuter la Forme située à la position laquelle avec la Forme située à la
      //   position laquelle + déplacement
      // Extrant :
      // - le tableau formes suite à la possible permutation
      //
      public static Forme[] ChangerProfondeur(Forme[] formes, int laquelle, int déplacement)
      {
         // Oups!
      }

      // -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
      // -- Vos ajouts personnels, le cas échéant, vont ci-dessous
      // -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-

   }
}